import {
	Alert,
	Box,
	Divider,
	SingleSelect,
	SingleSelectOption,
	Typography
} from "@strapi/design-system";
import { useCMEditViewDataManager } from "@strapi/helper-plugin";
import { AxiosError } from "axios";
import React, { useEffect, useState } from "react";

// eslint-disable-next-line no-restricted-imports
import axiosInstance from "../utils/axiosInstance";

type UserData = {
	key: string;
	id: number;
	email: string;
	firstname?: string;
	name: string;
	lastname?: string;
	username?: string;
	fullname?: string;
	code?: string;
};

const WorkflowCard = () => {
	const { slug, initialData } = useCMEditViewDataManager();

	const [listStages, setListStages] = useState([""]);
	const [stage, setStage] = useState("");
	const [workflowId, setWorkflowId] = useState();
	const [eligibileUsersAndRoles, setEligibileUsersAndRoles] = useState<UserData[]>([]);
	const [selectApprover, setSelectApprover] = useState("");
	const [selectAssignee, setSelectAssignee] = useState("");
	const [isStageToggleDisable, setIsStageToggleDisable] = useState(true);
	const [errMessage, setError] = useState<null | string>(null);

	async function getUserByKey(key) {
		const foundUser = eligibileUsersAndRoles.find((user) =>
			user.key === "user" ? user.email === key : user.code === key
		);
		return foundUser;
	}

	const getWorkflow = async (id) => {
		const { data: workflow } = await axiosInstance.get(`/review-workflow/${id}/${slug}`);
		return workflow.data;
	};

	const createWorkflow = async (value) => {
		try {
			const { data: response } = await axiosInstance.post("/review-workflow/create", {
				stage: `${value}`,
				type: slug,
				taskId: initialData.id
			});
			setWorkflowId(response.data.id);
			setStage(value);
		} catch (error) {
			let msg = `${error}`;
			if (error instanceof AxiosError) {
				msg = `${error.response?.data?.message}`;
			}
			setError(`${msg}`);
		}
	};

	const updateWorkflow = async (value, id) => {
		try {
			await axiosInstance.put(`/review-workflow/${id}`, { stage: value });
			setStage(value);
		} catch (error) {
			let msg = `${error}`;
			if (error instanceof AxiosError) {
				msg = `${error.response?.data?.message}`;
			}
			setError(`${msg}`);
		}
	};

	useEffect(() => {
		const fetchStagesAndEligibleUsers = async () => {
			try {
				const [{ data: allStages }, { data: eligibleUsers }, { data: eligibleRoles }] =
					await Promise.all([
						axiosInstance.get("/review-workflow/stages"),
						axiosInstance.get("/review-workflow/eligible/users"),
						axiosInstance.get("/review-workflow/eligible/roles")
					]);
				const rolesAndUsers = [...eligibleUsers.data, ...eligibleRoles.data];
				const rolesAndUsersWithKeys = rolesAndUsers.map((entry) => {
					const keyPrefix = entry.hasOwnProperty("email") ? "user" : "role";
					const uniqueKey = `${keyPrefix}`;
					return { key: uniqueKey, ...entry };
				});
				let workflowData;

				if (initialData.id) {
					setIsStageToggleDisable(false);
					workflowData = await getWorkflow(initialData.id);
				}
				if (workflowData && Object.keys(workflowData).length > 0) {
					const approverType = workflowData?.approverType;
					const assigneeType = workflowData?.assigneeType;
					setStage(workflowData.stage);
					setWorkflowId(workflowData.id);
					const normalizeApprover =
						workflowData?.approverType && capitalizeFirstLetter(`${approverType}`);
					const approverKey = normalizeApprover && `approverAs${normalizeApprover}`;
					const approverData = workflowData && workflowData[approverKey];
					if (approverData) {
						approverKey === "approverAsUser"
							? setSelectApprover(approverData.email)
							: setSelectApprover(approverData.code);
					}
					const normalizeAssignee =
						workflowData?.assigneeType && capitalizeFirstLetter(`${assigneeType}`);
					const assigneeKey = normalizeAssignee && `assigneeAs${normalizeAssignee}`;
					const assigneeData = workflowData && workflowData[assigneeKey];
					if (assigneeData) {
						assigneeKey === "assigneeAsUser"
							? setSelectAssignee(assigneeData.email)
							: setSelectAssignee(assigneeData.code);
					}
				}

				setEligibileUsersAndRoles(rolesAndUsersWithKeys);
				setListStages(allStages.data);
			} catch (error) {
				let msg = `${error}`;
				if (error instanceof AxiosError) {
					msg = `${error.response?.data?.message}`;
				}
				setError(msg);
			}
		};

		fetchStagesAndEligibleUsers();
	}, []);

	const approver = async (userKey) => {
		try {
			const user = await getUserByKey(userKey);

			const payload = {
				id: user?.id,
				approverType: user?.code ? "role" : "user"
			};

			await axiosInstance.put(`/review-workflow/${workflowId}/approver`, {
				approverId: Number(payload.id),
				approverType: payload.approverType
			});
			setSelectApprover(userKey);

			return;
		} catch (error) {
			let msg = `${error}`;
			if (error instanceof AxiosError) {
				msg = `${error.response?.data?.message}`;
			}
			setError(`${msg}`);
		}
	};

	const assignee = async (userKey) => {
		try {
			const user = await getUserByKey(userKey);

			const payload = {
				id: user?.id,
				assigneeType: user?.code ? "role" : "user"
			};

			await axiosInstance.put(`/review-workflow/${workflowId}/assignee`, {
				assigneeId: Number(payload.id),
				assigneeType: payload.assigneeType
			});

			setSelectAssignee(userKey);
			return;
		} catch (error) {
			let msg = `${error}`;
			if (error instanceof AxiosError) {
				msg = `${error.response?.data?.message}`;
			}
			setError(`${msg}`);
		}
	};
	const capitalizeFirstLetter = (str: string): string => {
		return str.charAt(0).toUpperCase() + str.slice(1);
	};

	const handleAlertClose = () => {
		setError(null);
	};

	return (
		<Box
			as="aside"
			aria-labelledby="additional-informations"
			background="neutral0"
			borderColor="neutral150"
			hasRadius
			paddingBottom={4}
			paddingLeft={4}
			paddingRight={4}
			paddingTop={3}
			shadow="tableShadow">
			<Typography
				variant="sigma"
				textColor="neutral600"
				id="additional-informations">
				Review Workflow
			</Typography>
			<Box
				paddingTop={2}
				paddingBottom={6}>
				<Box paddingBottom={2}>
					<Divider />
				</Box>
				<Box paddingTop={2}>
					{errMessage && (
						<Alert
							closeLabel="Close"
							onClose={handleAlertClose}
							title="Warning!"
							variant="danger">
							{errMessage}
						</Alert>
					)}
					{stage && (
						<SingleSelect
							label="Review Stages"
							value={stage}
							onChange={(event) => updateWorkflow(event, workflowId)}
							placeholder="Select Review stage">
							{listStages.map((stg) => (
								<SingleSelectOption value={stg}>{stg}</SingleSelectOption>
							))}
						</SingleSelect>
					)}
					{!stage && (
						<SingleSelect
							disabled={isStageToggleDisable}
							label="Review Stages"
							value={stage}
							onChange={createWorkflow}
							required
							placeholder="Select Review stage">
							{listStages.map((stg) => (
								<SingleSelectOption value={stg}>{stg}</SingleSelectOption>
							))}
						</SingleSelect>
					)}
					<SingleSelect
						disabled={stage === "" ? true : false}
						value={selectApprover}
						label="Approver"
						placeholder="Select Approver"
						onChange={(selectedUser) => approver(selectedUser)}>
						{eligibileUsersAndRoles.map((user) => (
							<SingleSelectOption
								value={
									user.key === "user" ? user.email : user.code
								}>{`${user.name}`}</SingleSelectOption>
						))}
					</SingleSelect>

					<SingleSelect
						disabled={stage === "" ? true : false}
						value={selectAssignee}
						label="Assignee"
						placeholder="Select Assignee"
						onChange={(selectedUserKey) => assignee(selectedUserKey)}>
						{eligibileUsersAndRoles.map((user) => (
							<SingleSelectOption
								value={
									user.key === "user" ? user.email : user.code
								}>{`${user.name}`}</SingleSelectOption>
						))}
					</SingleSelect>
				</Box>
			</Box>
		</Box>
	);
};

export default WorkflowCard;
