/* eslint-disable no-restricted-imports */
import type { Strapi } from "@strapi/strapi";
import { ZodError } from "zod";

import { capitalizeFirstLetter, findEntryById, isSuperAdmin, validateRequest } from "../utils";
import { CustomError, ForbiddenError, NotFoundError } from "../utils/error";
import { ErrorResponseObject, SuccessResponseObject } from "../utils/http";
import { approverBodySchema, workflowPathParamsSchema } from "./schema";

export default ({ strapi }: { strapi: Strapi }) => ({
	async update(ctx) {
		try {
			const { workflowId } = ctx.params;
			const { approverId, approverType } = ctx.request.body;
			await Promise.all([
				validateRequest(Number(workflowId), workflowPathParamsSchema),
				validateRequest(ctx.request.body, approverBodySchema)
			]);
			const entry = await findEntryById(workflowId);
			if (!entry) {
				throw new NotFoundError("No Entry found!");
			}

			const isUserSuperAdmin = isSuperAdmin(ctx.state.user.roles);
			const assignee = entry?.assigneeAsUser;
			const isUserSameAsAssignee = assignee && Number(assignee.id) === Number(approverId);

			if (isUserSameAsAssignee && !isUserSuperAdmin.length) {
				throw new ForbiddenError("APPROVER CAN'T be the same as ASSIGNEE.");
			}

			const approver = await strapi
				.plugin("review-workflow")
				.service("approverService")
				.update({
					approverId,
					workflowId,
					approverType: capitalizeFirstLetter(approverType),
					entry
				});
			ctx.status = 201;
			ctx.body = new SuccessResponseObject("Approver updated", approver);
			return ctx;
		} catch (err) {
			let status = 500;
			let message = "some error occurred"; // Don't expose db error and other errors internal to code like undefined etc.
			let data;
			if (err instanceof CustomError) {
				status = err.statusCode;
				message = err.message;
			}

			if (err instanceof ZodError) {
				status = 400;
				message = err.name;
				data = JSON.parse(err.message);
			}
			await strapi.plugin("sentry").service("sentry").sendError(JSON.stringify(err));
			ctx.status = status;

			ctx.body = new ErrorResponseObject(`${message}`, data);
			return ctx;
		}
	}
});
