/* eslint-disable no-restricted-imports */
import type { Strapi } from "@strapi/strapi";
import { ZodError } from "zod";

import {
	capitalizeFirstLetter,
	findEntryById,
	isRoleEligible,
	isSuperAdmin,
	isUserEligible,
	validateRequest
} from "../utils";
import { CustomError, ForbiddenError, NotFoundError } from "../utils/error";
import { ErrorResponseObject, SuccessResponseObject } from "../utils/http";
import { assignBodySchema, workflowPathParamsSchema } from "./schema";

export default ({ strapi }: { strapi: Strapi }) => ({
	async update(ctx) {
		try {
			const { workflowId } = ctx.params;
			const { assigneeId, assigneeType } = ctx.request.body;
			await Promise.all([
				await validateRequest(Number(workflowId), workflowPathParamsSchema),
				await validateRequest(ctx.request.body, assignBodySchema)
			]);
			const entry = await findEntryById(workflowId);
			if (!entry) {
				throw new NotFoundError("No Entry found!");
			}
			const isUserSuperAdmin = isSuperAdmin(ctx.state.user.roles);
			const isEntryApprover = entry.approverAsUser;
			const isUserSameAsApprover =
				isEntryApprover && Number(isEntryApprover.id) === Number(assigneeId);
			if (isUserSameAsApprover && !isUserSuperAdmin.length) {
				throw new ForbiddenError("Assignee CAN'T be the same as APPROVER.");
			}

			const assigneeRoleEligbible =
				`${assigneeType}`.toLowerCase().trim() === "role" && (await isRoleEligible(assigneeId));

			const userEligible =
				`${assigneeType}`.toLowerCase().trim() === "user" && (await isUserEligible(assigneeId));
			if (`${assigneeType}`.toLowerCase().trim() === "role" && !assigneeRoleEligbible) {
				throw new ForbiddenError("Role Not Eligible");
			}

			if (`${assigneeType}`.toLowerCase().trim() === "user" && !userEligible) {
				throw new ForbiddenError("User Not Eligible");
			}
			ctx.status = 201;
			const assignee = await strapi
				.plugin("review-workflow")
				.service("assigneeService")
				.update({
					assigneeId,
					workflowId,
					assigneeType: capitalizeFirstLetter(assigneeType),
					entry
				});
			ctx.body = new SuccessResponseObject("Assignee updated", assignee);
			return ctx;
		} catch (err) {
			let status = 500;
			let message = "some error occurred"; // Don't expose db error and other errors internal to code like undefined etc.
			let data;
			if (err instanceof CustomError) {
				status = err.statusCode;
				message = err.message;
			}

			if (err instanceof ZodError) {
				status = 400;
				message = err.name;
				data = JSON.parse(err.message);
			}

			await strapi.plugin("sentry").service("sentry").sendError(JSON.stringify(err));
			ctx.status = status;

			ctx.body = new ErrorResponseObject(`${message}`, data);
			return ctx;
		}
	}
});
