/* eslint-disable no-restricted-imports */
import type { Strapi } from "@strapi/strapi";

import {
	checkApproverAndAssignee,
	findEntryById,
	findOneEntry,
	handleErrors,
	validateRequest
} from "../utils";
import { ConflictError, NotFoundError } from "../utils/error";
import { SuccessResponseObject } from "../utils/http";
import {
	idPathParams,
	requestBodySchema,
	singleEntrySchema,
	updateRequestBodySchema
} from "./schema";

export default ({ strapi }: { strapi: Strapi }) => ({
	async allStages(ctx) {
		try {
			const stages = await strapi.plugin("review-workflow").service("workflowService").allStages();
			ctx.status = 200;
			ctx.body = new SuccessResponseObject("Successfully fetch all stages", stages);
		} catch (err) {
			const errResponse = await handleErrors(err);
			ctx.status = errResponse.status;
			ctx.body = errResponse.body;
			return ctx;
		}
	},

	async create(ctx) {
		try {
			await validateRequest(ctx.request.body, requestBodySchema);
			const { taskId, type } = ctx.request.body;
			const entryExist = await findOneEntry({ id: taskId, uid: type });

			if (Object.keys(entryExist).length !== 0) {
				throw new ConflictError("Duplicate not allowed.");
			}

			const createWorkflow = await strapi
				.plugin("review-workflow")
				.service("workflowService")
				.create(ctx.request.body, ctx.state.user);
			ctx.status = 201;
			ctx.body = new SuccessResponseObject("Workflow successfully created", createWorkflow);
			return ctx;
		} catch (err) {
			const errResponse = await handleErrors(err);
			ctx.status = errResponse.status;
			ctx.body = errResponse.body;
			return ctx;
		}
	},
	async update(ctx) {
		try {
			const data = ctx.request.body;
			const user = ctx.state.user;
			const { id } = ctx.params;
			await Promise.all([
				validateRequest(Number(id), idPathParams),
				validateRequest(data, updateRequestBodySchema)
			]);

			const entry = await findEntryById(id);
			if (!entry) {
				throw new NotFoundError("No entry found");
			}

			await checkApproverAndAssignee(entry, user, data);
			const updateWorkflow = await strapi
				.plugin("review-workflow")
				.service("workflowService")
				.updateEntry({ id, data, user, entry });

			ctx.status = 200;
			ctx.body = new SuccessResponseObject("Workflow updated", updateWorkflow);
			return ctx;
		} catch (err) {
			const errResponse = await handleErrors(err);
			ctx.status = errResponse.status;
			ctx.body = errResponse.body;
			return ctx;
		}
	},
	async delete(ctx) {
		try {
			const { id } = ctx.params;

			const deletWokflow = await strapi
				.plugin("review-workflow")
				.service("workflowService")
				.deleteEntry({ id });
			ctx.status = 204;
			ctx.body = new SuccessResponseObject("Workflow deleted", deletWokflow);
			return ctx;
		} catch (err) {
			const errResponse = await handleErrors(err);
			ctx.status = errResponse.status;
			ctx.body = errResponse.body;
			return ctx;
		}
	},
	async eligibleUsers(ctx) {
		try {
			const eligibleUsers = await strapi
				.plugin("review-workflow")
				.service("workflowService")
				.users();

			ctx.status = 200;
			ctx.body = new SuccessResponseObject("Eligible users fetched", eligibleUsers);

			return ctx;
		} catch (err) {
			const errResponse = await handleErrors(err);
			ctx.status = errResponse.status;
			ctx.body = errResponse.body;
			return ctx;
		}
	},

	async eligibleRoles(ctx) {
		try {
			const eligibleRoles = await strapi
				.plugin("review-workflow")
				.service("workflowService")
				.roles();

			ctx.status = 200;
			ctx.body = new SuccessResponseObject("Eligible Roles fetched", eligibleRoles);
			return ctx;
		} catch (err) {
			const errResponse = await handleErrors(err);
			ctx.status = errResponse.status;
			ctx.body = errResponse.body;
			return ctx;
		}
	},

	async getSingle(ctx) {
		try {
			const { target_id, target_type } = ctx.params;
			ctx.params.target_id = Number(ctx.params.target_id);
			await validateRequest(ctx.params, singleEntrySchema);

			const entry = await findOneEntry({ uid: target_type, id: target_id });

			ctx.status = 200;
			ctx.body = new SuccessResponseObject("Workflow successfully fetched", entry);
			return ctx;
		} catch (err) {
			const errResponse = await handleErrors(err);
			ctx.status = errResponse.status;
			ctx.body = errResponse.body;
			return ctx;
		}
	}
});
