import { ZodError } from "zod";

import { CustomError, ForbiddenError } from "./error";
import { ErrorResponseObject } from "./http";

export type Entry = {
	id?: string | number;
	uid: string;
};
export type Roles = {
	id: number;
	name: string;
};

export type UserDetails = {
	id: number;
	firstname: string;
	lastname?: string;
	email: string;
	username?: string | null;
};
export type RoleDetails = {
	id: number;
	name: string;
	code: string;
};

export type RoleEligibility = {
	id: number;
	name: string;
	code: string;
};

export type RoleResult = Role | undefined;

export type Role = {
	id: number;
	name: string;
	code: string;
};

export type EntryResponse = {
	id: number;
	stage: string;
	approverType?: string;
	assigneeType?: string;
	approverAsUser?: UserDetails;
	approverAsRole?: RoleDetails;
	assigneeAsUser?: UserDetails;
	assigneeAsRole?: RoleDetails;
};
export type UserOrRole = "user" | "role";
export const findOneEntry = async (params: Entry) => {
	const { uid, id } = params;

	if (uid && id) {
		const entry = await strapi.db.query("plugin::review-workflow.workflow").findOne({
			where: { target_id: id, target_type: uid },
			populate: ["approverAsUser", "approverAsRole", "assigneeAsUser", "assigneeAsRole"]
		});
		const response = {
			...entry
		};

		let normalizeAssignee;
		let normalizeApprover;

		if (entry?.approverAsUser) {
			normalizeApprover = mapUserRelationsData(entry.approverAsUser);
			response.approverAsUser = normalizeApprover;
		}

		if (entry?.assigneeAsUser) {
			normalizeAssignee = mapUserRelationsData(entry.assigneeAsUser);
			response.assigneeAsUser = normalizeAssignee;
		}

		return response ?? null;
	}
	const entry = await strapi.db.query("plugin::review-workflow.workflow").findOne({
		where: { target_type: uid },
		populate: ["approverAsUser", "approverAsRole", "assigneeAsUser", "assigneeAsRole"]
	});

	const response = { ...entry };
	if (response?.approverAsUser) {
		response.approverAsUser = mapUserRelationsData(entry.approverAsUser);
	}
	if (response?.assigneeAsUser) {
		response.assigneeAsUser = mapUserRelationsData(entry.assigneeAsUser);
	}
	return response ?? null;
};

export const findEntryById = async (id: number): Promise<any> => {
	return await strapi.entityService.findOne("plugin::review-workflow.workflow", id, {
		populate: {
			assigneeAsUser: {
				fields: ["id", "firstname", "lastname", "username", "email"]
			},
			assigneeAsRole: {
				fields: ["id", "name", "code"]
			},
			approverAsUser: {
				fields: ["id", "firstname", "lastname", "username", "email"]
			},
			approverAsRole: {
				fields: ["id", "name", "code"]
			}
		}
	});
};

export const validateRequest = async (payload, requestBodySchema) => {
	return await requestBodySchema.parse(payload);
};

export const isSuperAdmin = (role: Roles[]): Roles[] | [] => {
	return role.filter((role) => role.name === `Super Admin`.trim());
};

export const isApproveSameAsUser = (approverId: number, userId: number): boolean => {
	return approverId === userId;
};

export const isUserEligible = async (userId: number): Promise<UserDetails | null> => {
	const response = await strapi.entityService.findOne("admin::user", userId, {
		fields: ["id", "firstname", "lastname", "username", "email"],
		filters: { blocked: false, isActive: true }
	});

	return response as UserDetails;
};

export const isRoleEligible = async (id: number): Promise<Role | null> => {
	const role = await strapi.entityService.findOne("admin::role", id, {
		fields: ["id", "name", "code"]
	});

	return role as Role;
};

export const stageApproved = (stage): boolean => `${stage}`.trim().toLowerCase() === "approved";

export const mapUserData = (obj) => {
	return {
		id: obj.id,
		firstname: obj.approver?.firstname ?? obj.assignee?.firstname,
		lastname: obj.approver?.lastname ?? obj.assignee?.lastname,
		email: obj.approver?.email ?? obj.assignee?.email
	};
};

export const mapUserRelationsData = (obj) => {
	return {
		id: obj.id,
		name: `${obj.firstname} ${obj?.lastname}`,
		firstname: obj.firstname,
		lastname: obj?.lastname,
		email: obj.email
	};
};

export const mapRoleRelationsData = (obj) => {
	return {
		id: obj.id,
		name: obj.name,
		code: obj.code
	};
};

type Payload = {
	id: number;
	link: string;
	type: string;
};

export const unlinkRelation = async (data: Payload): Promise<boolean> => {
	const { id, link, type } = data;
	const unlink = await strapi.entityService?.update("plugin::review-workflow.workflow", id, {
		data: {
			[`${link}`]: null,
			[`${type}`]: null
		},
		populate: [`${link}`]
	});
	return unlink ? true : false;
};

export const capitalizeFirstLetter = (str: string): string => {
	return str.charAt(0).toUpperCase() + str.slice(1);
};

export const checkApproverAndAssignee = async (entry, user, data) => {
	const { approverAsRole, approverAsUser, assigneeAsUser, assigneeAsRole, approverType } = entry;
	const approver = approverAsRole ?? approverAsUser;
	const assignee = assigneeAsUser ?? assigneeAsRole;
	const isUserSuperAdmin = isSuperAdmin(user.roles);
	const isStageApproved = data?.stage && stageApproved(data.stage);
	if (isStageApproved && !(approver || assignee)) {
		throw new ForbiddenError("Assignee and Approver required.");
	}
	const userHasRolePermission =
		entry?.approverAsRole &&
		user.roles.filter(
			(role) =>
				`${role.name}`.toLowerCase().trim() === `${entry.approverAsRole.name}`.toLowerCase().trim()
		);

	if (
		approverType === "role" &&
		isStageApproved &&
		!isUserSuperAdmin.length &&
		!userHasRolePermission.length
	) {
		throw new ForbiddenError("Don't have permission to approve this task.");
	}
	if (
		approverType === "user" &&
		!isUserSuperAdmin.length &&
		isStageApproved &&
		entry.approverAsUser.id !== user.id
	) {
		throw new ForbiddenError("Don't have permission to approve this task.");
	}
};

export const handleErrors = async (err) => {
	let status = 500;
	let message = "some error occurred"; // Don't expose db error and other errors internal to code like undefined etc.
	let data;
	if (err instanceof CustomError) {
		status = err.statusCode;
		message = err.message;
	}

	if (err instanceof ZodError) {
		status = 400;
		message = err.name;
		data = JSON.parse(err.message);
	}

	if (process.env.NODE_ENV !== "test") {
		await strapi.plugin("sentry").service("sentry").sendError(JSON.stringify(err));
	}

	const body = new ErrorResponseObject(`${message}`, data);
	return { status, body };
};
